/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.job.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.annotation.Resource;

import net.sourceforge.kivu4j.job.domain.JobDto;
import net.sourceforge.kivu4j.job.domain.TriggerDto;
import net.sourceforge.kivu4j.utils.action.JSONActionSupport;
import net.sourceforge.kivu4j.utils.action.RequestMap;
import net.sourceforge.kivu4j.utils.action.ResponseEmpty;
import net.sourceforge.kivu4j.utils.action.ResponseList;
import net.sourceforge.kivu4j.utils.job.ScheduleManager;
import net.sourceforge.kivu4j.utils.lang.domain.NameValue;

import org.quartz.JobDetail;
import org.quartz.Trigger;


/**
 * 管理器请求。启动(startup)，关闭(shutdown)，暂停(standby)，激活(active)等等功能。
 * 
 *
 * @author Lucky Yang
 */
public class JobAction extends JSONActionSupport {
	@Resource
	private ScheduleManager scheduleManager;
	@Resource
	private JobResultFactory jobResultFactory;
	
	private NameValue getScheuleStandbyObject(){
		boolean rslt = this.scheduleManager.isStandby();
		return new NameValue("standby", rslt);
	}
	
	/**
	 * 启动，并查询状态。
	 * 
	 * @return
	 */
	public String startup(){
		try{
			this.scheduleManager.startup();
	
			this.setResponseData(new ResponseList<NameValue>(this.getScheuleStandbyObject()));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 关闭，并查询状态。
	 * 
	 * @return
	 */
	public String shutdown(){
		try{
			this.scheduleManager.shutdown();
			this.setResponseData(new ResponseList<NameValue>(this.getScheuleStandbyObject()));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 暂停，并查询状态。
	 * 
	 * @return
	 */
	public String standby(){
		try{
			this.scheduleManager.standby();
			this.setResponseData(new ResponseList<NameValue>(this.getScheuleStandbyObject()));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 激活，并查询状态。
	 * 
	 * @return
	 */
	public String active(){
		try{
			this.scheduleManager.active();
			this.setResponseData(new ResponseList<NameValue>(this.getScheuleStandbyObject()));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 暂停管理器的所有任务
	 * 
	 * @return
	 */
	public String pauseAll(){
		try{
			this.scheduleManager.pauseAll();
					
			this.setResponseData(new ResponseEmpty());
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 激活管理器的所有任务
	 * 
	 * @return
	 */
	public String resumeAll(){
		try{
			this.scheduleManager.resumeAll();
					
			this.setResponseData(new ResponseEmpty());
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 管理器元数据
	 * 
	 * @return
	 */
	public String loadSchedulerMetaData(){
		try{
			Collection<NameValue> rslt = this.scheduleManager.getMetaData();
			
			this.setResponseData(new ResponseList<NameValue>(rslt));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 管理器状态
	 * 
	 * @return
	 */
	public String isStandBy(){
		try{
			boolean rslt = this.scheduleManager.isStandby();
					
			this.setResponseData(new ResponseList<NameValue>(new NameValue("standby", rslt)));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 请求触发器列表。
	 * 
	 * @return
	 */
	public String loadTriggerByGroup(){
		try{
			RequestMap map = this.getRequestAsMap();
			String jobGroup = map.getStr("group");
			
			List<TriggerDto> rslt = new ArrayList<TriggerDto>();
			String[] jobNames = this.scheduleManager.getJobNames(jobGroup);
			for(String jonName : jobNames){
				Trigger[] triggers = this.scheduleManager.getTriggersOfJob(jonName, jobGroup);
				for(Trigger trigger : triggers){
					rslt.add(this.jobResultFactory.make(trigger));
				}
			}
			
			this.setResponseData(new ResponseList<TriggerDto>(rslt));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}	
	
	/**
	 * 激活触发器,并返回触发器信息
	 * 
	 * @return
	 */
	public String resumeTrigger(){
		try{
			RequestMap map = this.getRequestAsMap();
			String name = map.getStr("name");
			String group = map.getStr("group");
			
			this.scheduleManager.resumeTrigger(name, group);
			
			// 返回
			Trigger trigger = this.scheduleManager.getTrigger(name, group);
			this.setResponseData(new ResponseList<TriggerDto>(this.jobResultFactory.make(trigger)));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}	
	
	
	/**
	 * 暂停触发器,并返回触发器信息
	 * 
	 * @return
	 */
	public String pauseTrigger(){
		try{
			RequestMap map = this.getRequestAsMap();
			String name = map.getStr("name");
			String group = map.getStr("group");
			
			this.scheduleManager.pauseTrigger(name, group);
			
			// 返回
			Trigger trigger = this.scheduleManager.getTrigger(name, group);
			this.setResponseData(new ResponseList<TriggerDto>(this.jobResultFactory.make(trigger)));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}	
	
	
	/**
	 * 激活group
	 * 
	 * @return
	 */
	public String resumeGroup(){
		try{
			RequestMap map = this.getRequestAsMap();
			String group = map.getStr("group");
			
			this.scheduleManager.resumeGroup(group);
			
			this.setResponseData(new ResponseEmpty());
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 暂停group
	 * 
	 * @return
	 */
	public String pauseGroup(){
		try{
			RequestMap map = this.getRequestAsMap();
			String group = map.getStr("group");
			
			this.scheduleManager.pauseGroup(group);
			
			this.setResponseData(new ResponseEmpty());
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}
	
	/**
	 * 请求任务
	 * 
	 * @return
	 */
	public String loadJob(){
		try{
			RequestMap map = this.getRequestAsMap();
			String name = map.getStr("name");
			String group = map.getStr("group");
			
			JobDetail jobDetail = this.scheduleManager.getJobDetail(name, group);
			this.setResponseData(new ResponseList<JobDto>(this.jobResultFactory.make(jobDetail)));
		}catch(Exception e){
			this.handleException(e);
		}
		return SUCCESS;
	}	
}
