/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.job.services;

import java.util.Collection;

import javax.annotation.Resource;

import net.sourceforge.kivu4j.job.domain.Timetable;
import net.sourceforge.kivu4j.job.domain.TimetableRepository;
import net.sourceforge.kivu4j.utils.job.Consts;
import net.sourceforge.kivu4j.utils.job.JobException;
import net.sourceforge.kivu4j.utils.job.ScheduleManager;
import net.sourceforge.kivu4j.utils.lang.DateUtil;

import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.Trigger;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

/**
 * 实现
 *
 * @author Lucky Yang
 */
@Service("timetableManager")
public class TimetableManagerImpl implements TimetableManager {
	@Resource
	private ScheduleManager scheduleManager;
	@Resource
	private TimetableRepository timetableRepository;

	/**
	 * 检查有效性
	 * 
	 * @param timetable
	 */
	private void validate(Timetable timetable) {
		Assert.notNull(timetable, "'timetable' must not be null");
		Assert.notNull(timetable.getID(), "'timetable.id' must not be null");
		Assert.notNull(timetable.getClazz(), "'timetable.clazz' must not be null");
		Assert.notNull(timetable.getCronExpression(), "'timetable.cronExpression' must not be null");
	}
	
	/**
	 * 检查trigger，是否需要取消或者重新注册
	 * 
	 * @param trigger
	 */
	private void check(Trigger trigger) {
		Assert.notNull(trigger, "'trigger' must not be null");
		
		Timetable tt_in_trigger = (Timetable)trigger.getJobDataMap().get(Consts.TIMETABLE);
		
		if (tt_in_trigger == null){
			// 非法的trigger，取消
			this.scheduleManager.unscheduleJob(trigger.getName(), trigger.getGroup());
			return;
		}
		
		Timetable tt_in_database = this.timetableRepository.findById(tt_in_trigger.getID());
		if (tt_in_database == null || ! tt_in_database.isActive()){
			// Timetable在数据库中不存在，或者不活动
			this.scheduleManager.unscheduleJob(trigger.getName(), trigger.getGroup());
			return;
		}		
		
		// 重新发布
		Trigger newTrigger = this.createTrigger(trigger.getJobName(), trigger.getJobGroup(), tt_in_database);
		this.rescheduleJob(trigger, newTrigger);
	}

	/**
	 * 重新发布触发器。需要比较两个触发器的timetable时间。
	 * 
	 * @param trigger
	 * @param newTrigger
	 * @throws TriggerSuspectException
	 */
	private void rescheduleJob(Trigger trigger, Trigger newTrigger) {
		Timetable timetable = (Timetable)trigger.getJobDataMap().get(Consts.TIMETABLE);
		Timetable newTimetable = (Timetable)newTrigger.getJobDataMap().get(Consts.TIMETABLE);
		
		if (timetable == null)
			throw new TriggerSuspectException(trigger.getFullName());
		if (newTimetable == null)
			throw new TriggerSuspectException(newTrigger.getFullName());
		
		// 比较修改时间
		if (DateUtil.isABeforeB(timetable.getRealModifyOn(), newTimetable.getRealModifyOn())){	
			this.scheduleManager.rescheduleJob(trigger, newTrigger);		
		}	
	}

	/**
	 * 检查是否需要注册一个新的tirgger。
	 * 
	 * @param job
	 * @param timetable
	 */
	private void check(JobDetail job, Timetable timetable) {
		Assert.notNull(job, "'trigger' must not be null");
		Assert.notNull(timetable, "'timetable' must not be null");
		
		String jobName = job.getName();
		String jobGroup = job.getGroup();
		Trigger newTrigger = this.createTrigger(jobName, jobGroup, timetable);	
		
		//是否存在这个tirgger
		Trigger trigger = this.scheduleManager.getTrigger(newTrigger.getName(), newTrigger.getGroup());
		if (trigger == null){
			if (timetable.isActive()){
				// 全新的，需要发布
				this.scheduleManager.scheduleJob(newTrigger);
			}
		}else{
			// 触发器存在
			if (timetable.isActive()){
				// 重新发布
				this.rescheduleJob(trigger, newTrigger);
			}else{
				// 不活动，取消
				this.scheduleManager.unscheduleJob(trigger.getName(), trigger.getGroup());
			}
		}
	}

	/**
	 * 根据timetable实例，创建一个新的触发器
	 * 
	 * @param jobName
	 * @param jobGroup
	 * @param timetable
	 * @return
	 */
	private Trigger createTrigger(String jobName, String jobGroup,
			Timetable timetable) {
		this.validate(timetable);
		
		try {
			String trigger_name = timetable.getID() + "-" + jobName;
			CronTrigger trigger = new CronTrigger(trigger_name, jobGroup, timetable.getCronExpression());
			trigger.setJobName(jobName);
			trigger.setJobGroup(jobGroup);
			trigger.setDescription(timetable.getDescription());
			trigger.getJobDataMap().put(Consts.TIMETABLE, timetable);
			return trigger;
		} catch (Exception e) {
			throw new JobException(e);
		}
	}


	/**
	 * 循环处理所有的Timetable.
	 * @param job
	 */
	private void loopTimetable(JobDetail job) {
		Collection<Timetable> timetables = this.timetableRepository.findByClazz(job.getJobClass().getName());
		for(Timetable timetable:timetables){
			this.check(job, timetable);
		}
	}

	/**
	 * 循环处理所有的Trigger.
	 * @param job
	 */	
	private void loopTrigger(JobDetail job) {
		Trigger[] triggers = this.scheduleManager.getTriggersOfJob(job.getName(), job.getGroup());
		
		for(Trigger trigger:triggers){
			this.check(trigger);
		}
	}

	@Override
	public void validateTheJob(String jobName, String jobGroup) {
		Assert.notNull(jobName, "'jobName' must not be null");
		Assert.notNull(jobGroup, "'jobGroup' must not be null");
		
		JobDetail job = this.scheduleManager.getJobDetail(jobName, jobGroup);
		// 循环所有的Trigger
		this.loopTrigger(job);
		// 循环所有的Timetable
		this.loopTimetable(job);	
	}	
}
